#include "entier.hpp"

entier::entier(unsigned n)
{
	if (n==0)
	  {
		nb_chiffres = 1;
		decimale = new int[1];
		decimale[0] = 0;
	  }
	else
	  {
		int m = n;
		nb_chiffres = 0;
		// On compte le nombre de cases a reserver
		while (m!=0)
		  {
			m /= 10;
			nb_chiffres ++;
		  }
				
		decimale = new int[nb_chiffres];
		for (unsigned k=0; k<nb_chiffres; k++)
		  {
			decimale[k] = n%10; // On cacule les chiffres de n en base 10, decimale[k] correspond au chiffre des 10^k
			n /= 10;
		  }
	  }
	
  }



// Le constructeur par copie
entier::entier(entier const& n)
{
  decimale = new int[n.nb_chiffres];
  nb_chiffres = n.nb_chiffres;
  
  for (unsigned i=0; i<nb_chiffres; i++)
	decimale[i] = n.decimale[i];
}




entier & entier::operator=(entier const& n)
{
  if (&n != this) //On verifie que l'on n'efface pas l'objet que l'on veut recopier (si on ecrivait n=n)
	{
	  if (nb_chiffres != n.nb_chiffres)
		{
		  delete []decimale;
		  nb_chiffres = n.nb_chiffres;
		  decimale = new int[nb_chiffres];
		}
	  
	  for (unsigned i=0; i<nb_chiffres; i++)
		decimale[i] = n.decimale[i];
	}
  
  return *this;
}



entier& entier::operator+=(entier const &m)
{
  // On effectue l'addition en calculant les chiffres un par un, "comme a l'ecole primaire"

  // Si l'entier courant a moins de chiffres que m, on lui en rajoute
  if (nb_chiffres < m.nb_chiffres)
	{
	  int * decimale_temp = new int[m.nb_chiffres];

	  for (unsigned i=0; i<nb_chiffres; i++)
		decimale_temp[i] = decimale[i];
	  for (unsigned i=nb_chiffres; i<m.nb_chiffres; i++)
		decimale_temp[i] = 0;

	  nb_chiffres = m.nb_chiffres;
	  
	  delete [] decimale;
	  decimale = decimale_temp;
	}
  
  // On fait l'algorithme d'addition
  int retenue=0;
  for (unsigned i=0; i<nb_chiffres; i++)
	{
	  int m_decimale = i<m.nb_chiffres ? m.decimale[i] : 0;

	  int somme = decimale[i] + m_decimale + retenue;
	  
	  decimale[i] = somme % 10; // On fait la somme, en prenant en compte les eventuelles retenue.
	  retenue = somme / 10;
	}

  if (retenue == 1) // Si il reste une retenue a la fin, on doit ajouter un chiffre 1 au début.
	{
	  int * decimale_temp = new int[nb_chiffres+1];

	  for (unsigned i=0; i<nb_chiffres; i++)
		decimale_temp[i] = decimale[i];

	  decimale_temp[nb_chiffres] = 1;

	  nb_chiffres++;
	  
	  delete [] decimale;
	  decimale = decimale_temp;
	}
  
  return *this;
}


// On définit l'opérateur + à partir de l'opérateur += déjà défini.
entier operator+(entier const &n, entier const &m)
{
  entier N = n;
  N += m;
  return N;
}




entier operator*(entier const &n, entier const &m)
{
  entier prod(0);


  // On majore le nombre de chiffre du produit
  prod.nb_chiffres = n.nb_chiffres + m.nb_chiffres - 1;

  if ((n.decimale[n.nb_chiffres-1]+1) * (m.decimale[m.nb_chiffres-1]+1) >= 10)
	{
	  prod.nb_chiffres ++;
	}
  
  prod.decimale = new int[prod.nb_chiffres];

  int retenue = 0;
  for (unsigned i=0 ; i<prod.nb_chiffres ; i++)
	{
	  int somme = 0;
	  for (unsigned j=0; j<=i; j++)
		{
		  // Aux retenues pres, le chiffre des 10^i est la somme des produits a*b ou a est le chiffre des 10^j de n et b est le chiffre des 10^(i-j) de m.
		  int n_decimale = (j < n.nb_chiffres) ? n.decimale[j] : 0 ;
		  int m_decimale = (i-j < m.nb_chiffres) ? m.decimale[i-j] : 0 ;
		  somme += n_decimale * m_decimale;
		}
	  somme += retenue;
	  prod.decimale[i] = somme % 10;
	  retenue = somme / 10;
	}
  
  return prod;
}


entier& entier::operator*=(entier const &m)
{
  // Ecriture de *= a partir de *
  *this = *this * m;
  return *this;
}

std::ostream& operator<<(std::ostream &flux, entier const &n)
{
  for (int i=n.nb_chiffres-1; i>=0; i--) // On affiche les chiffres de n, en commencant par n.decimale[nb_chiffres-1]
	flux << n.decimale[i];
  return flux;
}
